@@ -110,9 +110,9 @@ type TSMFile interface {
 	// HasTombstones returns true if file contains values that have been deleted.
 	HasTombstones() bool
 
-	// TombstoneFiles returns the tombstone filestats if there are any tombstones
+	// TombstoneStats returns the tombstone filestats if there are any tombstones
 	// written for this file.
-	TombstoneFiles() []FileStat
+	TombstoneStats() TombstoneStat
 
 	// Close closes the underlying file resources.
 	Close() error
@@ -121,7 +121,7 @@ type TSMFile interface {
 	Size() uint32
 
 	// Rename renames the existing TSM file to a new name and replaces the mmap backing slice using the new
-	// file name.  Index and Reader state are not re-initialized.
+	// file name. Index and Reader state are not re-initialized.
 	Rename(path string) error
 
 	// Remove deletes the file from the filesystem.
@@ -205,6 +205,14 @@ type FileStat struct {
 	MinKey, MaxKey   []byte
 }
 
+// TombstoneStat holds information about a possible tombstone file on disk.
+type TombstoneStat struct {
+	TombstoneExists bool
+	Path            string
+	LastModified    int64
+	Size            uint32
+}
+
 // OverlapsTimeRange returns true if the time range of the file intersect min and max.
 func (f FileStat) OverlapsTimeRange(min, max int64) bool {
 	return f.MinTime <= max && f.MaxTime >= min
@@ -569,7 +577,7 @@ func (f *FileStore) Open() error {
 
 		// Accumulate file store size stats
 		atomic.AddInt64(&f.stats.DiskBytes, int64(res.r.Size()))
-		for _, ts := range res.r.TombstoneFiles() {
+		if ts := res.r.TombstoneStats(); ts.TombstoneExists {
 			atomic.AddInt64(&f.stats.DiskBytes, int64(ts.Size))
 		}
 
@@ -802,8 +810,8 @@ func (f *FileStore) replace(oldFiles, newFiles []string, updatedFn func(r []TSMF
 					return err
 				}
 
-				for _, t := range file.TombstoneFiles() {
-					if err := f.obs.FileUnlinking(t.Path); err != nil {
+				if ts := file.TombstoneStats(); ts.TombstoneExists {
+					if err := f.obs.FileUnlinking(ts.Path); err != nil {
 						return err
 					}
 				}
@@ -821,8 +829,8 @@ func (f *FileStore) replace(oldFiles, newFiles []string, updatedFn func(r []TSMF
 				if file.InUse() {
 					// Copy all the tombstones related to this TSM file
 					var deletes []string
-					for _, t := range file.TombstoneFiles() {
-						deletes = append(deletes, t.Path)
+					if ts := file.TombstoneStats(); ts.TombstoneExists {
+						deletes = append(deletes, ts.Path)
 					}
 
 					// Rename the TSM file used by this reader
@@ -884,10 +892,9 @@ func (f *FileStore) replace(oldFiles, newFiles []string, updatedFn func(r []TSMF
 	var totalSize int64
 	for _, file := range f.files {
 		totalSize += int64(file.Size())
-		for _, ts := range file.TombstoneFiles() {
+		if ts := file.TombstoneStats(); ts.TombstoneExists {
 			totalSize += int64(ts.Size)
 		}
-
 	}
 	atomic.StoreInt64(&f.stats.DiskBytes, totalSize)
 
@@ -1048,7 +1055,7 @@ func (f *FileStore) MakeSnapshotLinks(destPath string, files []TSMFile) error {
 		if err := os.Link(tsmf.Path(), newpath); err != nil {
 			return fmt.Errorf("error creating tsm hard link: %q", err)
 		}
-		for _, tf := range tsmf.TombstoneFiles() {
+		if tf := tsmf.TombstoneStats(); tf.TombstoneExists {
 			newpath := filepath.Join(destPath, filepath.Base(tf.Path))
 			if err := os.Link(tf.Path, newpath); err != nil {
 				return fmt.Errorf("error creating tombstone hard link: %q", err)
@@ -1091,7 +1098,7 @@ func (f *FileStore) CreateSnapshot() (string, error) {
 		if err := os.RemoveAll(tmpPath); err != nil {
 			// report if, for some reason, we couldn't remove our temporary
 			// directory.
-			return "", fmt.Errorf("CreateSnapshot() failed to create links and failed to remove temporary direcotry %v: %w", tmpPath, err)
+			return "", fmt.Errorf("CreateSnapshot() failed to create links and failed to remove temporary direcotry %v: %v", tmpPath, err)
 		}
 		return "", err
 	}
